function cost = antenna_cost(x)
  // Access global file paths
  global nec_filename results_csv log_csv;
  
  // Access global target values
  global VSWR0 G0 FBH0;
  
  // Access global weigths
  global VSWR_W G_W FBH_W;

  // Element spacings
  d1 = x(1);
  d2 = x(2);

  global trial_count;
  trial_count = trial_count + 1;
  
  // Modify the .NEC file with new values
  modify_nec_file(nec_filename, d1, d2);

  // Run AN-SOFengine.exe
  dos('start /wait /min C:\AN-SOFengine\AN-SOFengine.exe');

  // Read output gain from CSV
  [vswr, gain, fbh] = read_antenna_results(results_csv);

  if isempty(vswr) | isnan(vswr) | ...
     isempty(gain) | isnan(gain) | ...
     isempty(fbh)  | isnan(fbh) then
     trial_count = trial_count - 1;
     cost = 1e6; // A large penalty value so optimizer discards this trial
     return;
  end

  // Display valid results
  disp(msprintf("%4d  | %5.4f | %5.4f | %5.2f | %5.2f | %5.2f", ... 
                trial_count, d1, d2, vswr, gain, fbh));
  
  // Append results to CSV file
  fid = mopen(log_csv, "at"); // Open in append mode
  mfprintf(fid, "%d,%5.4f,%5.4f,%5.2f,%5.2f,%5.2f\n", ...
           trial_count, d1, d2, vswr, gain, fbh);
  mclose(fid);

  // Cost function: Minimize weigthed relative error
  cost = (VSWR_W* abs(vswr - VSWR0)/VSWR0 + ...
          G_W* abs(gain - G0)/G0 + ...
          FBH_W* abs(fbh - FBH0)/FBH0) / ...
          (VSWR_W + G_W + FBH_W);
endfunction

function modify_nec_file(filename, d1, d2)
  Seg = 7;
  antenna = [
  GW(1, Seg, -0.25, 0, 0, 0.25, 0, 0, 0.001)
  GW(2, Seg, -0.238, d1, 0, 0.238, d1, 0, 0.001)
  GW(3, Seg, -0.226, d1+d2, 0, 0.226, d1+d2, 0, 0.001)
  GE(0)
  FR(0, 1, 300, 0.0)
  EX(0, 2, (Seg+1)/2, 1.4142136, 0)
  RP(37, 73, 0, 0, 5, 5, 1)
  ];

  // Write modified .NEC file
  mputl(antenna, filename);
endfunction

function [vswr, gain, fbh] = read_antenna_results(filename)
  // Read CSV file
  [data, c] = csvRead(filename, ';', [], [], [], [], [], 3);
    
  // VSWR is in the 7th column
  vswr = data(1, 7);
  // Gain is in the 10th column
  gain = data(1, 10);
  // F/B(H) is in the 15th column
  fbh = data(1, 15);
endfunction

global trial_count;
trial_count = 0;

// Global file paths
global nec_filename results_csv log_csv;
  
// Define file paths
nec_filename = "C:\AN-SOF\yagi3.nec";
results_csv = "C:\AN-SOF\yagi3_Results.csv";
log_csv = "C:\AN-SOF\GAoptimizer_log.csv"; // CSV log file for trial results

// Set directory for AN-SOFengine.exe
chdir("C:\AN-SOFengine\");

// Display table header with trial results
disp("Trial |   d1   |   d2   |  VSWR | G(dBi)| F/B(H)");
disp("------------------------------------------------");

fid = mopen(log_csv, "wt");
mfprintf(fid, "Trial,d1,d2,VSWR,Gain(dBi),F/B(H)\n");
mclose(fid);

// Define target values and weights
global VSWR0 G0 FBH0;
global VSWR_W G_W FBH_W;

VSWR0 = 1.1;   // Target VSWR
G0 = 9.0;      // Target gain in dBi
FBH0 = 15.0;   // Target F/B(H) in dB

VSWR_W = 80;  // VSWR weight %
G_W = 50;     // Gain weight %
FBH_W = 75;   // F/B(H) weight %

// Define bounds for d1 and d2
lower_bounds = [0.1; 0.1]; // Minimum spacing (meters)
upper_bounds = [0.4; 0.4]; // Maximum spacing (meters)

// GA parameters
pop_size = 10;   // Population size
max_gen = 3;     // Number of generations
crossover_rate = 0.7;
mutation_rate = 0.1;
ga_params = init_param();
ga_params = add_param(ga_params,"dimension",2);
ga_params = add_param(ga_params,"minbound",lower_bounds);
ga_params = add_param(ga_params,"maxbound",upper_bounds);

// Run the Genetic Algorithm
[optim_result, optim_cost] = optim_ga(antenna_cost, ...
               pop_size, max_gen, mutation_rate, crossover_rate, %T, ga_params);
               
antenna_cost(optim_result(1));

// exec("C:\Scilab Projects\GAoptimizer.sce", -1);

